%% BYOM, with stdDEB model: byom_showcal.m
%
% *Table of contents*

%% About
% * Author     : Tjalling Jager
% * Date       : April 2022
% * Web support: <http://www.debtox.info/byom.html>
%
% BYOM is a General framework for simulating model systems in terms of
% ordinary differential equations (ODEs). The model itself needs to be
% specified in <derivatives.html derivatives.m>, and <call_deri.html
% call_deri.m> may need to be modified to the particular problem as well.
% The files in the engine directory are needed for fitting and plotting.
% Results are shown on screen but also saved to a log file (results.out).
%
% *The model:* standard DEB model in terms of powers (energy fluxes in J/d)
% and body length. In this package, toxicant stress is included. The model
% equations are those from the 'Tromso coffee mug' ;-). Only change is that
% the ODE for structure is phrased in volumetric length rather than volume.
% The TKTD model is lifted from DEBtox2019. The 2023 publication of Jager
% et al in Ecological Modelling contains the full details:
% <https://doi.org/10.1016/j.ecolmodel.2022.110187>.
%
% *This script:* This works generically, as all species- and
% compound-specific information is loaded from the MAT file. This script
% allows the user to replot a previous calibration. The data set is loaded,
% which is save in the calibration file. This script works generically, as
% all species- and compound-specific information is loaded from the MAT
% file. Furthermore, it is now even largely model independent, as glo and
% X0mat are also saved during the calibration, and loaded here. All
% DEB-based TKTD analyses should work with this script.
% 
%  Copyright (c) 2012-2023, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(1) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 0; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% The data set will be loaded from the MAT file by select_pred. This
% requires that DATA was saved in the calibration script!

%% Call the Matlab GUI open-file element to load profile and MAT file

[conf_type,~,par] = select_pred([3 1 1]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters as well as all settings used to generate the MAT
% file (in glo) including the exposure profile. This only works when the
% MAT file was generated by BYOM v.6.0 BETA 7 or newer as older versions
% won't save glo. When using an old MAT file (pre BYOM v6), make sure to
% set glo.Tbp to the correct value if it needs to be >0. Also, the data set
% is loaded, when it was saved in the calibration file with the same name
% (..._DATA.mat).

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios. All identifiers and initial values, as part of
% the matrix X0mat, are also loaded by select_pred already. Feel free to
% overwrite them with other values if needed.
% 
% NOTE: the glo.locD etc. have all been loaded from the MAT file by select_pred.

%% Initial values for the model parameters
% All parameters have been loaded from file using select_pred. All global
% parameters, as part of the structure glo, are also loaded by select_pred
% already. Feel free to overwrite them with other values.
  
%% Time vector and labels for plots
% Specify what to plot. If time vector glo.t is not specified, a default is
% constructed, based on the data set.

% All plot labels have been loaded from file using select_pred.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Calculations and plotting
% Here, the function is called that will do the calculation and the plotting.
% Options for the plotting can be set using opt_plot (see prelim_checks.m).
% Options for the optimsation routine can be set using opt_optim. Options
% for the ODE solver are part of the global glo. 
% 
% NOTE: for this package, the options useode and eventson in glo will not
% be functional: the ODE solver is always used, and the events function as
% well. Further, note that glo.stiff and glo.break_time are loaded from the
% MAT file as well; best to use the saved ones to get the exact same output
% as during calibration.
 
opt_optim.fit    = 1; % fit the parameters (1), or don't (0)
opt_plot.bw      = 1; % if set to 1, plots in black and white with different plot symbols
opt_plot.annot   = 2; % annotations in multiplot for fits: 1) box with parameter estimates 2) single legend
opt_plot.repls   = 0; % set to 1 to plot replicates, 0 to plot mean responses
basenm_rem       = glo.basenm; % remember basename as automatic_runs may modify it!

% These settings for opt_optim allow you to see the parspace plot, without refitting.
opt_optim.type     = 4; % optimisation method: 1) default simplex, 4) parspace explorer
opt_optim.fit      = 1; % fit the parameters (1), or don't (0)
opt_optim.ps_saved = 1; % use saved set for parameter-space explorer (1) or not (0);
opt_plot.statsup   = []; % vector with states to suppress in plotting fits
opt_plot.limax     = 1; % if set to 1, limit axes to the data set for each stage

par_out = calc_optim(par,opt_optim); % start the optimisation
% Call calc_optim if you want to see the parspace plot.

% Optionally: plot with calc_and_plot, which shows all treatments in the
% same plot, and also shows the additional state variables for reserve and
% maturity. The reset option will work from BYOM v 6.5.
opt_plot.bw      = 1; % if set to 1, plots in black and white with different plot symbols
opt_plot.cn      = 1; % if set to 1, connect model line to points (only for bw=1)
opt_plot.plt_rst = 1; % reset symbols/colours for each additional data set
calc_and_plot(par_out,opt_plot); % calculate model lines and plot them

%% Plot results with confidence intervals
% The following code can be used to make plots with confidence intervals.
% Options for confidence bounds on model curves can be set using opt_conf
% (see prelim_checks). The plot_tktd function makes multiplots for the
% effects data, which are more readable when plotting with various
% intervals.

opt_conf.type     = conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
opt_conf.lim_set  = 2; % use limited set of n_lim points (1) or outer hull (2, not for Bayes) to create CIs
opt_tktd.repls    = 0; % plot individual replicates (1) or means (0)
opt_tktd.transf   = 1; % set to 1 to calculate means and SEs including transformations
opt_tktd.set_ctrl = 1; % set to 1 to use separate control per data set for the dotted lines
opt_tktd.lim_data = 1; % set to 1 to limit axes to data

opt_tktd.max_exp  = 0; % set to 1 to maximise exposure/damage plots on exposure rather than damage
opt_tktd.obspred  = 1; % plot predicted-observed plots (1) or not (0)
opt_tktd.sppe     = 1; % set to 1 to calculate SPPEs (relative error at end of test)
% Note: this last option will produce something like the SPPE, but requires
% an update of the BYOM engine to Beta 2 (opt_tktd.obspred must be set to
% 1 to calculate SPPE and other metrics).

fit_tox = [1 1 3]; % use both controls
% change X0mat to avoid plotting controls not used for fitting (assumes
% that regular control has identifier 0 and solvent control 0.1)
switch fit_tox(3)
    case 1 % remove solvent control
        X0mat(:,X0mat(1,:)==0.1) = [];
    case 2 % remove regular control
        X0mat(:,X0mat(1,:)==0) = [];
    case 3
        % leave all in
end

% Option 1: use basic and tox parameters from the saved set.
plot_tktd([],opt_tktd,opt_conf); 
% Note that first input is empty: this is for the parameter structure,
% which is then obtained from the saved sample.
